@@ -1,6 +1,6 @@
 // Copyright 2008 Gautam Sewani
 // Copyright 2013 Paul A. Bristow
-
+// Copyright 2024 Matt Borland
 // Use, modification and distribution are subject to the
 // Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt
@@ -11,12 +11,19 @@
 #  pragma warning (disable : 4512) // assignment operator could not be generated.
 #endif
 
-#include <boost/config.hpp>
-#ifndef BOOST_NO_EXCEPTIONS
-#define BOOST_MATH_UNDERFLOW_ERROR_POLICY throw_on_error
-#define BOOST_MATH_OVERFLOW_ERROR_POLICY throw_on_error
+#ifdef SYCL_LANGUAGE_VERSION
+#include "sycl/sycl.hpp"
 #endif
-#include <boost/math/tools/test.hpp>
+
+#include <boost/config.hpp>
+#include <boost/math/tools/config.hpp>
+
+#if !defined(BOOST_NO_EXCEPTIONS) && !defined(BOOST_MATH_NO_EXCEPTIONS)
+#  define BOOST_MATH_UNDERFLOW_ERROR_POLICY throw_on_error
+#  define BOOST_MATH_OVERFLOW_ERROR_POLICY throw_on_error
+#endif
+
+#include "../include_private/boost/math/tools/test.hpp"
 #include <boost/math/concepts/real_concept.hpp> // for real_concept
 #include <boost/math/distributions/logistic.hpp>
     using boost::math::logistic_distribution;
@@ -26,26 +33,44 @@
 #include <boost/test/tools/floating_point_comparison.hpp>
 #include "test_out_of_range.hpp"
 
+#include <type_traits>
 #include <iostream>
    using std::cout;
    using std::endl;
    using std::setprecision;
 
 template <class RealType>
-void test_spot(RealType location, RealType scale, RealType x, RealType p, RealType q, RealType tolerance)
+void test_spot(RealType location, RealType scale, RealType x, RealType p, RealType q, RealType logp, RealType logq, RealType tolerance, RealType logtolerance, RealType logtoleranceq)
 {
+   if (std::is_same<RealType, long double>::value || std::is_same<RealType, boost::math::concepts::real_concept>::value)
+   {
+      logtoleranceq *= 100;
+   }
+   
    BOOST_CHECK_CLOSE(
       ::boost::math::cdf(
       logistic_distribution<RealType>(location,scale),      
       x),
       p,
       tolerance); // %
+   BOOST_CHECK_CLOSE(
+      ::boost::math::logcdf(
+      logistic_distribution<RealType>(location,scale),      
+      x),
+      logp,
+      logtolerance); // %
    BOOST_CHECK_CLOSE(
       ::boost::math::cdf(
       complement(logistic_distribution<RealType>(location,scale),      
       x)),
       q,
       tolerance); // %
+   BOOST_CHECK_CLOSE(
+      ::boost::math::logcdf(
+      complement(logistic_distribution<RealType>(location,scale),      
+      x)),
+      log(q),
+      logtoleranceq); // %
    if(p < 0.999)
    {
       BOOST_CHECK_CLOSE(
@@ -62,7 +87,7 @@ void test_spot(RealType location, RealType scale, RealType x, RealType p, RealTy
          complement(logistic_distribution<RealType>(location,scale),      
          q)),
          x,
-         2 * tolerance); // %
+         3 * tolerance); // %
    }
 }
 
@@ -76,6 +101,14 @@ void test_spots(RealType T)
    RealType tolerance = (std::max)(
       static_cast<RealType>(1e-33L),
       boost::math::tools::epsilon<RealType>());
+
+   #if !(defined(__amd64__) || defined(__x86_64__) || defined(_M_AMD64))
+   if (std::is_same<RealType, long double>::value || std::is_same<RealType, boost::math::concepts::real_concept>::value)
+   {
+      tolerance = std::numeric_limits<double>::epsilon();
+   } 
+   #endif
+
    cout<<"Absolute tolerance:"<<tolerance<<endl;
 
    tolerance *= 50 * 100; 
@@ -93,23 +126,35 @@ void test_spots(RealType T)
       static_cast<RealType>(0.1L), // x
       static_cast<RealType>(0.141851064900487789594278108470953L), // p
       static_cast<RealType>(0.858148935099512210405721891529047L), //q
+      static_cast<RealType>(-1.95297761052607413459937686381969L), // log(p)
+      static_cast<RealType>(-0.15297761052607413459937686381969L), // log(q)
+      tolerance,
+      tolerance,
       tolerance);
-
+   
    test_spot(
       static_cast<RealType>(5), // location
       static_cast<RealType>(2), // scale
       static_cast<RealType>(3.123123123L),//x 
       static_cast<RealType>(0.281215878622547904873088053477813L), // p
       static_cast<RealType>(0.718784121377452095126911946522187L), //q
+      static_cast<RealType>(-1.26863265327659162571930803425160L), // log(p)
+      static_cast<RealType>(-0.33019421477659162571930803425160L), // log(q)
+      tolerance,
+      tolerance,
       tolerance);
-
+   
    test_spot(
       static_cast<RealType>(1.2345L), // location
       static_cast<RealType>(0.12345L), // scale
       static_cast<RealType>(3.123123123L),//x
       static_cast<RealType>(0.999999773084685079723328282229357L), // p
       static_cast<RealType>(2.26915314920276671717770643005212e-7L), //q
-      tolerance);
+      static_cast<RealType>(-2.2691534066556063906815875163479345e-7L), // Log(p) Edge case
+      static_cast<RealType>(-15.298688951095170556204624487357L), // log(q)
+      tolerance,
+      tolerance * 100000,
+      tolerance * 100000);
 
 
    //High probability
@@ -119,7 +164,11 @@ void test_spots(RealType T)
       static_cast<RealType>(10), // x
       static_cast<RealType>(0.99999998477002048723965105559179L), // p  
       static_cast<RealType>(1.5229979512760348944408208801237e-8L), //q
-      tolerance);
+      static_cast<RealType>(-1.5229979628736488101501003766470705e-8L), // log(p)
+      static_cast<RealType>(-18.000000015229979628736488101501L), // Log(q)
+      tolerance,
+      tolerance * 100000,
+      tolerance * 100000);
 
    //negative x
    test_spot(
@@ -128,17 +177,23 @@ void test_spots(RealType T)
       static_cast<RealType>(-0.1L), // scale
       static_cast<RealType>(0.0724264853615177178439235061476928L), // p
       static_cast<RealType>(0.927573514638482282156076493852307L), //q
+      static_cast<RealType>(-2.6251832265757900570194868863670116L), // log(p)
+      static_cast<RealType>(-0.075183226575790057019486886367011L), // log(q)
+      tolerance,
+      tolerance,
       tolerance);
 
-
    test_spot(
       static_cast<RealType>(5), // location
       static_cast<RealType>(2), // scale
       static_cast<RealType>(-20), // x
       static_cast<RealType>(3.72663928418656138608800947863869e-6L), // p
       static_cast<RealType>(0.999996273360715813438613911990521L), //q
-      tolerance);
-
+      static_cast<RealType>(-12.500003726646228123990312715291263L), // log(p)
+      static_cast<RealType>(-3.7266462281239903127152912633031e-6L), // Log(q)
+      tolerance,
+      tolerance,
+      tolerance * 100000);
 
    // Test value to check cancellation error in straight/complemented quantile.
    // The subtraction in the formula location-scale*log term introduces catastrophic
@@ -150,7 +205,11 @@ void test_spots(RealType T)
       static_cast<RealType>(-0.00125796420642514024493852425918807L),// x
       static_cast<RealType>(0.7L), // p
       static_cast<RealType>(0.3L), //q
-      80*tolerance);   
+      static_cast<RealType>(-0.35667494393873237891263871124118448L), // Log(p)
+      static_cast<RealType>(-1.2039728043259359926227462177618L), // log(q)
+      80*tolerance,
+      tolerance,
+      tolerance);   
 
    test_spot(
       static_cast<RealType>(1.2345L), // location
@@ -158,17 +217,23 @@ void test_spots(RealType T)
       static_cast<RealType>(0.0012345L), // x
       static_cast<RealType>(0.0000458541039469413343331170952855318L), // p
       static_cast<RealType>(0.999954145896053058665666882904714L), //q
-      80*tolerance);
-
-
-
+      static_cast<RealType>(-9.9900458551552785044234447480622748L), // Log(p)
+      static_cast<RealType>(-0.000045855155278504423444748062274193L), // log(q)
+      80*tolerance,
+      tolerance,
+      tolerance * 100000);
+   
    test_spot(
       static_cast<RealType>(5L), // location
       static_cast<RealType>(2L), // scale
       static_cast<RealType>(0.0012345L), // x
       static_cast<RealType>(0.0759014628704232983512906076564256L), // p
       static_cast<RealType>(0.924098537129576701648709392343574L), //q
-      80*tolerance);
+      static_cast<RealType>(-2.5783193211111713937468666265459702L), // Log(p)
+      static_cast<RealType>(-0.078936571111171393746866626545970L), // log(q)
+      80*tolerance,
+      tolerance,
+      tolerance);
 
    //negative location
    test_spot(
@@ -177,7 +242,12 @@ void test_spots(RealType T)
       static_cast<RealType>(3), // x
       static_cast<RealType>(0.999999999999999999999999984171276L), // p
       static_cast<RealType>(1.58287236765203121622150720373972e-26L), //q
-      tolerance);
+      static_cast<RealType>(-1.58287236765203121622150720373972e-26L), // Log(p)
+      static_cast<RealType>(-59.407971267074894017899199262178L), // log(q)
+      tolerance,
+      tolerance * 100000,
+      tolerance * 10000000);
+
    //PDF Testing
    BOOST_CHECK_CLOSE(
       ::boost::math::pdf(
@@ -371,7 +441,7 @@ BOOST_AUTO_TEST_CASE( test_main )
   test_spots(0.0); // Test double. OK at decdigits 7, tolerance = 1e07 %
 #ifndef BOOST_MATH_NO_LONG_DOUBLE_MATH_FUNCTIONS
   test_spots(0.0L); // Test long double.
-#if !BOOST_WORKAROUND(BOOST_BORLANDC, BOOST_TESTED_AT(0x582))
+#if !BOOST_WORKAROUND(BOOST_BORLANDC, BOOST_TESTED_AT(0x582)) && !defined(BOOST_MATH_NO_REAL_CONCEPT_TESTS)
   test_spots(boost::math::concepts::real_concept(0.)); // Test real concept.
 #endif
 #else
