@@ -549,15 +549,6 @@ void test_float_type()
    example15<Num>();
    example16<Num>();
    example17<Num>();
-   /*
-   example18<Num>();
-   example19<Num>();
-   example20<Num>();
-   example21<Num>();
-   example22<Num>();
-   example23<Num>();
-   example24<Num>();
-   */
 }
 
 template <class Num>
@@ -572,11 +563,9 @@ void test_float_type_2()
    example18<Num>();
    example19<Num>();
    example20<Num>();
+#if EIGEN_VERSION_AT_LEAST(3, 4, 0) || (BOOST_CXX_VERSION <= 201703L)
    example21<Num>();
-
-   //example22<Num>();
-   //example23<Num>();
-   //example24<Num>();
+#endif
 }
 
 template <class Num>
@@ -593,6 +582,29 @@ void test_float_type_3()
    example24<Num>();
 }
 
+template <class Num>
+void complex_eg_1()
+{
+#if EIGEN_VERSION_AT_LEAST(3, 4, 90)
+   using namespace std;
+   Matrix<Num, 3, 2> m = Matrix<Num, 3, 2>::Random(3, 2);
+   cout << "Here is the matrix m:" << endl
+        << m << endl;
+   JacobiSVD<Matrix<Num, 3, 2>, ComputeThinU | ComputeThinV> svd(m);
+   cout << "Its singular values are:" << endl
+        << svd.singularValues() << endl;
+   cout << "Its left singular vectors are the columns of the thin U matrix:" << endl
+        << svd.matrixU() << endl;
+   cout << "Its right singular vectors are the columns of the thin V matrix:" << endl
+        << svd.matrixV() << endl;
+   Vector<Num, 3> rhs(1, 0, 0);
+   cout << "Now consider this rhs vector:" << endl
+        << rhs << endl;
+   cout << "A least-squares solution of m*x = rhs is:" << endl
+        << svd.solve(rhs) << endl;
+#endif
+}
+
 template <class Num>
 void test_complex_type()
 {
@@ -620,8 +632,12 @@ void test_complex_type()
    example18a<Num>();
    example19<Num>();
    example20<Num>();
+#if EIGEN_VERSION_AT_LEAST(3, 4, 0) || (BOOST_CXX_VERSION <= 201703L)
    example21<Num>();
+#endif
    example22<Num>();
    // example23<Num>();  //requires comparisons.
    example24<Num>();
+
+   complex_eg_1<Num>();
 }
