@@ -1,7 +1,6 @@
-//Copyright (c) 2008-2017 Emil Dotchevski and Reverge Studios, Inc.
-
-//Distributed under the Boost Software License, Version 1.0. (See accompanying
-//file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
+// Copyright 2008-2024 Emil Dotchevski and Reverge Studios, Inc.
+// Distributed under the Boost Software License, Version 1.0. (See accompanying
+// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
 #include "boost/throw_exception.hpp"
 #include "boost/exception/info.hpp"
@@ -28,7 +27,7 @@
 #define INCLUDE_MATH "boost/qvm/math.hpp"
 #define INCLUDE_THROW_EXCEPTION "boost/qvm/throw_exception.hpp"
 #define INCLUDE_ERROR "boost/qvm/error.hpp"
-#define INCLUDE_INLINE "boost/qvm/inline.hpp"
+#define INCLUDE_CONFIG "boost/qvm/config.hpp"
 #define INCLUDE_M_TRAITS "boost/qvm/mat_traits.hpp"
 #define INCLUDE_V_TRAITS "boost/qvm/vec_traits.hpp"
 #define INCLUDE_Q_TRAITS "boost/qvm/quat_traits.hpp"
@@ -143,12 +142,10 @@ namespace
                 "#ifndef " << include_guard << NL
                 "#define " << include_guard << NL
                 NL
-                "/// Copyright (c) 2008-2021 Emil Dotchevski and Reverge Studios, Inc." NL
-                NL
-                "/// Distributed under the Boost Software License, Version 1.0. (See accompanying" NL
-                "/// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)" NL
-                NL
-                "/// This file was generated by a program. Do not edit manually." NL
+                "// Copyright 2008-2024 Emil Dotchevski and Reverge Studios, Inc." NL
+                "// This file was generated by a program. Do not edit manually." NL
+                "// Distributed under the Boost Software License, Version 1.0. (See accompanying" NL
+                "// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)" NL
                 NL
                 ;
             for( std::set<std::string>::const_iterator i=includes_.begin(),e=includes_.end(); i!=e; ++i )
@@ -206,7 +203,7 @@ namespace
         out.require_include(INCLUDE_DEDUCE_M);
         out.stream() <<
             "template <class A,class B>" NL
-            "BOOST_QVM_INLINE_OPERATIONS" NL
+            "BOOST_QVM_CONSTEXPR BOOST_QVM_INLINE_OPERATIONS" NL
             "typename lazy_enable_if_c<" NL
             TAB1 "mat_traits<A>::rows=="<<r<<" && mat_traits<B>::rows=="<<r<<" &&" NL
             TAB1 "mat_traits<A>::cols=="<<c<<" && mat_traits<B>::cols=="<<c<<"," NL
@@ -225,7 +222,7 @@ namespace
         out.require_include(INCLUDE_DEDUCE_M);
         out.stream()<<
             "template <class A,class B>" NL
-            "BOOST_QVM_INLINE_OPERATIONS" NL
+            "BOOST_QVM_CONSTEXPR BOOST_QVM_INLINE_OPERATIONS" NL
             "typename lazy_enable_if_c<" NL
             TAB1 "mat_traits<A>::rows=="<<m<<" && mat_traits<B>::rows=="<<n<<" &&" NL
             TAB1 "mat_traits<A>::cols=="<<n<<" && mat_traits<B>::cols=="<<p<<"," NL
@@ -243,7 +240,7 @@ namespace
         out.require_include(INCLUDE_DEDUCE_V);
         out.stream()<<
             "template <class A,class B>" NL
-            "BOOST_QVM_INLINE_OPERATIONS" NL
+            "BOOST_QVM_CONSTEXPR BOOST_QVM_INLINE_OPERATIONS" NL
             "typename lazy_enable_if_c<" NL
             TAB1 "mat_traits<A>::rows=="<<r<<" && mat_traits<A>::cols=="<<c<<" &&" NL
             TAB1 "vec_traits<B>::dim=="<<c<<"," NL
@@ -261,7 +258,7 @@ namespace
         out.require_include(INCLUDE_DEDUCE_V);
         out.stream()<<
             "template <class A,class B>" NL
-            "BOOST_QVM_INLINE_OPERATIONS" NL
+            "BOOST_QVM_CONSTEXPR BOOST_QVM_INLINE_OPERATIONS" NL
             "typename lazy_enable_if_c<" NL
             TAB1 "mat_traits<B>::rows=="<<r<<" && mat_traits<B>::cols=="<<c<<" &&" NL
             TAB1 "vec_traits<A>::dim=="<<c<<"," NL
@@ -278,7 +275,7 @@ namespace
         out.require_include(INCLUDE_DEDUCE_V);
         out.stream()<<
             "template <class A,class B>" NL
-            "BOOST_QVM_INLINE_OPERATIONS" NL
+            "BOOST_QVM_CONSTEXPR BOOST_QVM_INLINE_OPERATIONS" NL
             "typename lazy_enable_if_c<" NL
             TAB1 "vec_traits<A>::dim=="<<d<<" && vec_traits<B>::dim=="<<d<<"," NL
             TAB1 "deduce_vec2<A,B,"<<d<<"> >::type" NL
@@ -294,7 +291,7 @@ namespace
         assert(!name.empty());
         out.stream()<<
             "template <class A,class B>" NL
-            "BOOST_QVM_INLINE_OPERATIONS" NL
+            "BOOST_QVM_CONSTEXPR BOOST_QVM_INLINE_OPERATIONS" NL
             "typename enable_if_c<" NL
             TAB1 "mat_traits<A>::rows=="<<r<<" && mat_traits<B>::rows=="<<r<<" &&" NL
             TAB1 "mat_traits<A>::cols=="<<c<<" && mat_traits<B>::cols=="<<c<<"," NL
@@ -310,7 +307,7 @@ namespace
         assert(!name.empty());
         out.stream()<<
             "template <class A,class B>" NL
-            "BOOST_QVM_INLINE_OPERATIONS" NL
+            "BOOST_QVM_CONSTEXPR BOOST_QVM_INLINE_OPERATIONS" NL
             "typename enable_if_c<" NL
             TAB1 "vec_traits<A>::dim=="<<d<<" && vec_traits<B>::dim=="<<d<<"," NL
             "bool>::type" NL
@@ -326,7 +323,7 @@ namespace
         assert(!name.empty());
         out.stream()<<
             "template <class A,class B>" NL
-            "BOOST_QVM_INLINE_OPERATIONS" NL
+            "BOOST_QVM_CONSTEXPR BOOST_QVM_INLINE_OPERATIONS" NL
             "typename enable_if_c<" NL
             TAB1 "mat_traits<A>::rows=="<<r<<" && mat_traits<B>::rows=="<<r<<" &&" NL
             TAB1 "mat_traits<A>::cols=="<<c<<" && mat_traits<B>::cols=="<<c<<"," NL
@@ -342,7 +339,7 @@ namespace
         assert(!name.empty());
         out.stream()<<
             "template <class A,class B>" NL
-            "BOOST_QVM_INLINE_OPERATIONS" NL
+            "BOOST_QVM_CONSTEXPR BOOST_QVM_INLINE_OPERATIONS" NL
             "typename enable_if_c<" NL
             TAB1 "vec_traits<A>::dim=="<<d<<" && vec_traits<B>::dim=="<<d<<"," NL
             TAB1 "A &>::type" NL
@@ -358,7 +355,7 @@ namespace
         assert(!name.empty());
         out.stream()<<
             "template <class A>" NL
-            "BOOST_QVM_INLINE_OPERATIONS" NL
+            "BOOST_QVM_CONSTEXPR BOOST_QVM_INLINE_OPERATIONS" NL
             "typename enable_if_c<" NL
             TAB1 "mat_traits<A>::rows=="<<r<<" && mat_traits<A>::cols=="<<c<<"," NL
             TAB1 "typename mat_traits<A>::scalar_type>::type" NL
@@ -374,7 +371,7 @@ namespace
         out.require_include(INCLUDE_DEDUCE_S);
         out.stream()<<
             "template <class A,class B>" NL
-            "BOOST_QVM_INLINE_OPERATIONS" NL
+            "BOOST_QVM_CONSTEXPR BOOST_QVM_INLINE_OPERATIONS" NL
             "typename lazy_enable_if_c<" NL
             TAB1 "vec_traits<A>::dim=="<<d<<" && vec_traits<B>::dim=="<<d<<"," NL
             TAB1 "deduce_scalar<typename vec_traits<A>::scalar_type,typename vec_traits<B>::scalar_type> >::type" NL
@@ -389,7 +386,7 @@ namespace
         assert(!name.empty());
         out.stream()<<
             "template <class A>" NL
-            "BOOST_QVM_INLINE_OPERATIONS" NL
+            "BOOST_QVM_CONSTEXPR BOOST_QVM_INLINE_OPERATIONS" NL
             "typename enable_if_c<" NL
             TAB1 "is_vec<A>::value && vec_traits<A>::dim=="<<d<<"," NL
             TAB1 "typename vec_traits<A>::scalar_type>::type" NL
@@ -406,7 +403,7 @@ namespace
         out.require_include(INCLUDE_DEDUCE_M);
         out.stream()<<
             "template <class A>" NL
-            "BOOST_QVM_INLINE_OPERATIONS" NL
+            "BOOST_QVM_CONSTEXPR BOOST_QVM_INLINE_OPERATIONS" NL
             "typename lazy_enable_if_c<" NL
             TAB1 "mat_traits<A>::rows=="<<r<<" && mat_traits<A>::cols=="<<c<<"," NL
             TAB1 "deduce_mat<A> >::type" NL
@@ -422,7 +419,7 @@ namespace
         out.require_include(INCLUDE_DEDUCE_V);
         out.stream()<<
             "template <class A>" NL
-            "BOOST_QVM_INLINE_OPERATIONS" NL
+            "BOOST_QVM_CONSTEXPR BOOST_QVM_INLINE_OPERATIONS" NL
             "typename lazy_enable_if_c<" NL
             TAB1 "vec_traits<A>::dim=="<<d<<"," NL
             TAB1 "deduce_vec<A> >::type" NL
@@ -437,7 +434,7 @@ namespace
         assert(!name.empty());
         out.stream()<<
             "template <class R,class A>" NL
-            "BOOST_QVM_INLINE_OPERATIONS" NL
+            "BOOST_QVM_CONSTEXPR BOOST_QVM_INLINE_OPERATIONS" NL
             "typename enable_if_c<" NL
             TAB1 "is_vec<A>::value &&" NL
             TAB1 "vec_traits<R>::dim=="<<d<<" && vec_traits<A>::dim=="<<d<<"," NL
@@ -455,7 +452,7 @@ namespace
         out.require_include(INCLUDE_DEDUCE_M);
         out.stream()<<
             "template <class A,class B>" NL
-            "BOOST_QVM_INLINE_OPERATIONS" NL
+            "BOOST_QVM_CONSTEXPR BOOST_QVM_INLINE_OPERATIONS" NL
             "typename lazy_enable_if_c<" NL
             TAB1 "mat_traits<A>::rows=="<<r<<" && mat_traits<A>::cols=="<<c<<" && is_scalar<B>::value," NL
             TAB1 "deduce_mat2<A,B,mat_traits<A>::rows,mat_traits<A>::cols> >::type" NL
@@ -472,7 +469,7 @@ namespace
         out.require_include(INCLUDE_DEDUCE_M);
         out.stream()<<
             "template <class A,class B>" NL
-            "BOOST_QVM_INLINE_OPERATIONS" NL
+            "BOOST_QVM_CONSTEXPR BOOST_QVM_INLINE_OPERATIONS" NL
             "typename lazy_enable_if_c<" NL
             TAB1 "is_scalar<A>::value && mat_traits<B>::rows=="<<r<<" && mat_traits<B>::cols=="<<c<<"," NL
             TAB1 "deduce_mat2<A,B,mat_traits<B>::rows,mat_traits<B>::cols> >::type" NL
@@ -488,7 +485,7 @@ namespace
         out.require_include(INCLUDE_DEDUCE_V);
         out.stream()<<
             "template <class A,class B>" NL
-            "BOOST_QVM_INLINE_OPERATIONS" NL
+            "BOOST_QVM_CONSTEXPR BOOST_QVM_INLINE_OPERATIONS" NL
             "typename lazy_enable_if_c<" NL
             TAB1 "vec_traits<A>::dim=="<<d<<" && is_scalar<B>::value," NL
             TAB1 "deduce_vec2<A,B,vec_traits<A>::dim> >::type" NL
@@ -504,7 +501,7 @@ namespace
         out.require_include(INCLUDE_DEDUCE_V);
         out.stream()<<
             "template <class A,class B>" NL
-            "BOOST_QVM_INLINE_OPERATIONS" NL
+            "BOOST_QVM_CONSTEXPR BOOST_QVM_INLINE_OPERATIONS" NL
             "typename lazy_enable_if_c<" NL
             TAB1 "is_scalar<A>::value && vec_traits<B>::dim=="<<d<<"," NL
             TAB1 "deduce_vec2<A,B,vec_traits<B>::dim> >::type" NL
@@ -520,7 +517,7 @@ namespace
         assert(!name.empty());
         out.stream()<<
             "template <class A,class B>" NL
-            "BOOST_QVM_INLINE_OPERATIONS" NL
+            "BOOST_QVM_CONSTEXPR BOOST_QVM_INLINE_OPERATIONS" NL
             "typename enable_if_c<" NL
             TAB1 "mat_traits<A>::rows=="<<r<<" && mat_traits<A>::cols=="<<c<<" && is_scalar<B>::value," NL
             TAB1 "A &>::type" NL
@@ -535,7 +532,7 @@ namespace
         assert(!name.empty());
         out.stream()<<
             "template <class A,class  B>" NL
-            "BOOST_QVM_INLINE_OPERATIONS" NL
+            "BOOST_QVM_CONSTEXPR BOOST_QVM_INLINE_OPERATIONS" NL
             "typename enable_if_c<" NL
             TAB1 "vec_traits<A>::dim=="<<d<<" && is_scalar<B>::value," NL
             TAB1 "A &>::type" NL
@@ -671,14 +668,14 @@ namespace
         for( int i=0; i!=m; ++i )
             for( int j=0; j!=p; ++j )
                 {
-                g<<TAB1 "mat_traits<R>::template write_element<"<<i<<","<<j<<">(r)=";
+                g<<TAB1 "write_mat_element<"<<i<<","<<j<<">(r,";
                 for( int k=0; k!=n; ++k )
                     {
                     if( k )
                         g<<'+';
                     g<<'a'<<i<<k<<"*b"<<k<<j;
                     }
-                g<<";" NL;
+                g<<");" NL;
                 }
         g<<
             TAB1 "return r;" NL
@@ -707,14 +704,14 @@ namespace
         for( int i=0; i!=d; ++i )
             for( int j=0; j!=d; ++j )
                 {
-                g<<TAB1 "mat_traits<A>::template write_element<"<<i<<","<<j<<">(a)=";
+                g<<TAB1 "write_mat_element<"<<i<<","<<j<<">(a,";
                 for( int k=0; k!=d; ++k )
                     {
                     if( k )
                         g<<'+';
                     g<<'a'<<i<<k<<"*b"<<k<<j;
                     }
-                g<<";" NL;
+                g<<");" NL;
                 }
         g<<
             TAB1 "return a;" NL
@@ -729,7 +726,7 @@ namespace
         assert(r>0);
         assert(c>0);
         header_vr_ma_vb_mult(out,r,c,"operator*");
-        out.require_include(INCLUDE_INLINE);
+        out.require_include(INCLUDE_CONFIG);
         out.require_include(INCLUDE_V_TRAITS);
         out.require_include(INCLUDE_M_TRAITS);
         out.require_include(INCLUDE_ENABLE_IF);
@@ -752,14 +749,14 @@ namespace
             ;
         for( int i=0; i!=r; ++i )
             {
-            g<<TAB1 "vec_traits<R>::template write_element<"<<i<<">(r)=";
+            g<<TAB1 "write_vec_element<"<<i<<">(r,";
             for( int j=0; j!=c; ++j )
                 {
                 if( j )
                     g<<'+';
                 g<<'a'<<i<<j<<"*b"<<j;
                 }
-            g<<";" NL;
+            g<<");" NL;
             }
         g<<
             TAB1 "return r;" NL
@@ -774,7 +771,7 @@ namespace
         assert(r>0);
         assert(c>0);
         header_vr_va_mb_mult(out,r,c,"operator*");
-        out.require_include(INCLUDE_INLINE);
+        out.require_include(INCLUDE_CONFIG);
         out.require_include(INCLUDE_V_TRAITS);
         out.require_include(INCLUDE_M_TRAITS);
         out.require_include(INCLUDE_ENABLE_IF);
@@ -797,14 +794,14 @@ namespace
             ;
         for( int i=0; i!=c; ++i )
             {
-            g<<TAB1 "vec_traits<R>::template write_element<"<<i<<">(r)=";
+            g<<TAB1 "write_vec_element<"<<i<<">(r,";
             for( int j=0; j!=r; ++j )
                 {
                 if( j )
                     g<<'+';
                 g<<'a'<<j<<"*b"<<j<<i;
                 }
-            g<<";" NL;
+            g<<");" NL;
             }
         g<<
             TAB1 "return r;" NL
@@ -827,7 +824,7 @@ namespace
             TAB1 "R r;" NL
             ;
         for( int i=0; i!=d; ++i )
-            g<<TAB1 "vec_traits<R>::template write_element<"<<i<<">(r)=vec_traits<A>::template read_element<"<<i<<">(a)"<<op<<"vec_traits<B>::template read_element<"<<i<<">(b);" NL;
+            g<<TAB1 "write_vec_element<"<<i<<">(r,vec_traits<A>::template read_element<"<<i<<">(a)"<<op<<"vec_traits<B>::template read_element<"<<i<<">(b));" NL;
         g<<
             TAB1 "return r;" NL
             TAB1 "}" NL
@@ -931,7 +928,7 @@ namespace
             ;
         for( int i=0; i!=r; ++i )
             for( int j=0; j!=c; ++j )
-                g<<TAB1 "mat_traits<R>::template write_element<"<<i<<","<<j<<">(r)=mat_traits<A>::template read_element<"<<i<<","<<j<<">(a)"<<op<<"mat_traits<B>::template read_element<"<<i<<","<<j<<">(b);" NL;
+                g<<TAB1 "write_mat_element<"<<i<<","<<j<<">(r,mat_traits<A>::template read_element<"<<i<<","<<j<<">(a)"<<op<<"mat_traits<B>::template read_element<"<<i<<","<<j<<">(b));" NL;
         g<<
             TAB1 "return r;" NL
             TAB1 "}" NL
@@ -948,7 +945,7 @@ namespace
         g<<TAB1 "{" NL;
         for( int i=0; i!=r; ++i )
             for( int j=0; j!=c; ++j )
-                g<<TAB1 "mat_traits<A>::template write_element<"<<i<<","<<j<<">(a)"<<op<<"mat_traits<B>::template read_element<"<<i<<","<<j<<">(b);" NL;
+                g<<TAB1 "write_mat_element<"<<i<<","<<j<<">(a,mat_traits<A>::template read_element<"<<i<<","<<j<<">(a)"<<op<<"mat_traits<B>::template read_element<"<<i<<","<<j<<">(b));" NL;
         g<<
             TAB1 "return a;" NL
             TAB1 "}" NL
@@ -964,7 +961,7 @@ namespace
         std::ostream & g=out.stream();
         g<<TAB1 "{" NL;
         for( int i=0; i!=d; ++i )
-            g<<TAB1 "vec_traits<A>::template write_element<"<<i<<">(a)"<<op<<"vec_traits<B>::template read_element<"<<i<<">(b);" NL;
+            g<<TAB1 "write_vec_element<"<<i<<">(a,vec_traits<A>::template read_element<"<<i<<">(a)"<<op<<"vec_traits<B>::template read_element<"<<i<<">(b));" NL;
         g<<
             TAB1 "return a;" NL
             TAB1 "}" NL
@@ -986,7 +983,7 @@ namespace
             ;
         for( int i=0; i!=r; ++i )
             for( int j=0; j!=c; ++j )
-                g<<TAB1 "mat_traits<R>::template write_element<"<<i<<","<<j<<">(r)="<<op<<"mat_traits<A>::template read_element<"<<i<<","<<j<<">(a);" NL;
+                g<<TAB1 "write_mat_element<"<<i<<","<<j<<">(r,"<<op<<"mat_traits<A>::template read_element<"<<i<<","<<j<<">(a));" NL;
         g<<
             TAB1 "return r;" NL
             TAB1 "}" NL
@@ -1007,7 +1004,7 @@ namespace
             TAB1 "R r;" NL
             ;
         for( int i=0; i!=d; ++i )
-            g<<TAB1 "vec_traits<R>::template write_element<"<<i<<">(r)="<<op<<"vec_traits<A>::template read_element<"<<i<<">(a);" NL;
+            g<<TAB1 "write_vec_element<"<<i<<">(r,"<<op<<"vec_traits<A>::template read_element<"<<i<<">(a));" NL;
         g<<
             TAB1 "return r;" NL
             TAB1 "}" NL
@@ -1030,7 +1027,7 @@ namespace
             ;
         for( int i=0; i!=r; ++i )
             for( int j=0; j!=c; ++j )
-                g<<TAB1 "mat_traits<R>::template write_element<"<<i<<","<<j<<">(r)=mat_traits<A>::template read_element<"<<i<<","<<j<<">(a)"<<op<<"b;" NL;
+                g<<TAB1 "write_mat_element<"<<i<<","<<j<<">(r,mat_traits<A>::template read_element<"<<i<<","<<j<<">(a)"<<op<<"b);" NL;
         g<<
             TAB1 "return r;" NL
             TAB1 "}" NL
@@ -1053,7 +1050,7 @@ namespace
             ;
         for( int i=0; i!=r; ++i )
             for( int j=0; j!=c; ++j )
-                g<<TAB1 "mat_traits<R>::template write_element<"<<i<<","<<j<<">(r)=a"<<op<<"mat_traits<B>::template read_element<"<<i<<","<<j<<">(b);" NL;
+                g<<TAB1 "write_mat_element<"<<i<<","<<j<<">(r,a"<<op<<"mat_traits<B>::template read_element<"<<i<<","<<j<<">(b));" NL;
         g<<
             TAB1 "return r;" NL
             TAB1 "}" NL
@@ -1074,7 +1071,7 @@ namespace
             TAB1 "R r;" NL
             ;
         for( int i=0; i!=d; ++i )
-            g<<TAB1 "vec_traits<R>::template write_element<"<<i<<">(r)=vec_traits<A>::template read_element<"<<i<<">(a)"<<op<<"b;" NL;
+            g<<TAB1 "write_vec_element<"<<i<<">(r,vec_traits<A>::template read_element<"<<i<<">(a)"<<op<<"b);" NL;
         g<<
             TAB1 "return r;" NL
             TAB1 "}" NL
@@ -1095,7 +1092,7 @@ namespace
             TAB1 "R r;" NL
             ;
         for( int i=0; i!=d; ++i )
-            g<<TAB1 "vec_traits<R>::template write_element<"<<i<<">(r)=a"<<op<<"vec_traits<B>::template read_element<"<<i<<">(b);" NL;
+            g<<TAB1 "write_vec_element<"<<i<<">(r,a"<<op<<"vec_traits<B>::template read_element<"<<i<<">(b));" NL;
         g<<
             TAB1 "return r;" NL
             TAB1 "}" NL
@@ -1114,7 +1111,7 @@ namespace
             ;
         for( int i=0; i!=r; ++i )
             for( int j=0; j!=c; ++j )
-                g<<TAB1 "mat_traits<A>::template write_element<"<<i<<","<<j<<">(a)"<<op<<"b;" NL;
+                g<<TAB1 "write_mat_element<"<<i<<","<<j<<">(a,mat_traits<A>::template read_element<"<<i<<","<<j<<">(a)"<<op<<"b);" NL;
         g<<
             TAB1 "return a;" NL
             TAB1 "}" NL
@@ -1132,7 +1129,7 @@ namespace
             TAB1 "{" NL
             ;
         for( int i=0; i!=d; ++i )
-            g<<TAB1 "vec_traits<A>::template write_element<"<<i<<">(a)"<<op<<"b;" NL;
+            g<<TAB1 "write_vec_element<"<<i<<">(a, vec_traits<A>::template read_element<"<<i<<">(a)"<<op<<"b);" NL;
         g<<
             TAB1 "return a;" NL
             TAB1 "}" NL
@@ -1145,13 +1142,13 @@ namespace
         {
         header_ma_mb_same_size(out,r,c,"assign");
         out.require_include(INCLUDE_M_TRAITS);
-        out.require_include(INCLUDE_INLINE);
+        out.require_include(INCLUDE_CONFIG);
         out.require_include(INCLUDE_ENABLE_IF);
         std::ostream & g=out.stream();
         g<<TAB1 "{" NL;
         for( int i=0; i!=r; ++i )
             for( int j=0; j!=c; ++j )
-                g<<TAB1 "mat_traits<A>::template write_element<"<<i<<","<<j<<">(a)=mat_traits<B>::template read_element<"<<i<<","<<j<<">(b);" NL;
+                g<<TAB1 "write_mat_element<"<<i<<","<<j<<">(a,mat_traits<B>::template read_element<"<<i<<","<<j<<">(b));" NL;
         g<<
             TAB1 "return a;" NL
             TAB1 "}" NL
@@ -1164,12 +1161,12 @@ namespace
         {
         header_va_vb_same_size(out,d,"assign");
         out.require_include(INCLUDE_V_TRAITS);
-        out.require_include(INCLUDE_INLINE);
+        out.require_include(INCLUDE_CONFIG);
         out.require_include(INCLUDE_ENABLE_IF);
         std::ostream & g=out.stream();
         g<<TAB1 "{" NL;
         for( int i=0; i!=d; ++i )
-            g<<TAB1 "vec_traits<A>::template write_element<"<<i<<">(a)=vec_traits<B>::template read_element<"<<i<<">(b);" NL;
+            g<<TAB1 "write_vec_element<"<<i<<">(a,vec_traits<B>::template read_element<"<<i<<">(b));" NL;
         g<<
             TAB1 "return a;" NL
             TAB1 "}" NL
@@ -1188,7 +1185,7 @@ namespace
         std::ostream & g=out.stream();
         g<<
             "template <class R,class A>" NL
-            "BOOST_QVM_INLINE_OPERATIONS" NL
+            "BOOST_QVM_CONSTEXPR BOOST_QVM_INLINE_OPERATIONS" NL
             "typename enable_if_c<" NL
             TAB1 "mat_traits<R>::rows=="<<r<<" && mat_traits<A>::rows=="<<r<<" &&" NL
             TAB1 "mat_traits<R>::cols=="<<c<<" && mat_traits<A>::cols=="<<c<<"," NL
@@ -1199,7 +1196,7 @@ namespace
             ;
         for( int i=0; i!=r; ++i )
             for( int j=0; j!=c; ++j )
-                g<<TAB1 "mat_traits<R>::template write_element<"<<i<<","<<j<<">(r) = mat_traits<A>::template read_element<"<<i<<","<<j<<">(a);" NL;
+                g<<TAB1 "write_mat_element<"<<i<<","<<j<<">(r,mat_traits<A>::template read_element<"<<i<<","<<j<<">(a));" NL;
         g<<
             TAB1 "return r;" NL
             TAB1 "}" NL
@@ -1209,7 +1206,7 @@ namespace
             g<<
                 NL
                 "template <class R,class A>" NL
-                "BOOST_QVM_INLINE" NL
+                "BOOST_QVM_CONSTEXPR BOOST_QVM_INLINE" NL
                 "typename enable_if_c<" NL
                 TAB1 "is_mat<R>::value && is_quat<A>::value &&" NL
                 TAB1 "mat_traits<R>::rows=="<<r<<" && mat_traits<R>::cols=="<<c<<"," NL
@@ -1234,29 +1231,29 @@ namespace
                 TAB1 "T const one = scalar_traits<T>::value(1);" NL
                 TAB1 "T const two = one+one;" NL
                 TAB1 "R r;" NL
-                TAB1 "mat_traits<R>::template write_element<0,0>(r) = one - two*(cc+dd);" NL
-                TAB1 "mat_traits<R>::template write_element<0,1>(r) = two*(bc-ad);" NL
-                TAB1 "mat_traits<R>::template write_element<0,2>(r) = two*(bd+ac);" NL
+                TAB1 "write_mat_element<0,0>(r,one - two*(cc+dd));" NL
+                TAB1 "write_mat_element<0,1>(r,two*(bc-ad));" NL
+                TAB1 "write_mat_element<0,2>(r,two*(bd+ac));" NL
                 ;
             for( int i=3; i!=c; ++i )
-                g<<TAB1 "mat_traits<R>::template write_element<0,"<<i<<">(r) = zero;" NL;
+                g<<TAB1 "write_mat_element<0,"<<i<<">(r,zero);" NL;
             g<<
-                TAB1 "mat_traits<R>::template write_element<1,0>(r) = two*(bc+ad);" NL
-                TAB1 "mat_traits<R>::template write_element<1,1>(r) = one - two*(bb+dd);" NL
-                TAB1 "mat_traits<R>::template write_element<1,2>(r) = two*(cd-ab);" NL
+                TAB1 "write_mat_element<1,0>(r,two*(bc+ad));" NL
+                TAB1 "write_mat_element<1,1>(r,one - two*(bb+dd));" NL
+                TAB1 "write_mat_element<1,2>(r,two*(cd-ab));" NL
                 ;
             for( int i=3; i!=c; ++i )
-                g<<TAB1 "mat_traits<R>::template write_element<1,"<<i<<">(r) = zero;" NL;
+                g<<TAB1 "write_mat_element<1,"<<i<<">(r,zero);" NL;
             g<<
-                TAB1 "mat_traits<R>::template write_element<2,0>(r) = two*(bd-ac);" NL
-                TAB1 "mat_traits<R>::template write_element<2,1>(r) = two*(cd+ab);" NL
-                TAB1 "mat_traits<R>::template write_element<2,2>(r) = one - two*(bb+cc);" NL
+                TAB1 "write_mat_element<2,0>(r,two*(bd-ac));" NL
+                TAB1 "write_mat_element<2,1>(r,two*(cd+ab));" NL
+                TAB1 "write_mat_element<2,2>(r,one - two*(bb+cc));" NL
                 ;
             for( int i=3; i!=c; ++i )
-                g<<TAB1 "mat_traits<R>::template write_element<2,"<<i<<">(r) = zero;" NL;
+                g<<TAB1 "write_mat_element<2,"<<i<<">(r,zero);" NL;
             for( int i=3; i!=r; ++i )
                 for( int j=0; j!=c; ++j )
-                    g<<TAB1 "mat_traits<R>::template write_element<"<<i<<","<<j<<">(r) = "<<(i==j?"one":"zero")<<";" NL;
+                    g<<TAB1 "write_mat_element<"<<i<<","<<j<<">(r,"<<(i==j?"one":"zero")<<");" NL;
             g<<
                 TAB1 "return r;" NL
                 TAB1 "}" NL
@@ -1274,7 +1271,7 @@ namespace
             TAB1 "R r;" NL
             ;
         for( int i=0; i!=d; ++i )
-            g<<TAB1 "vec_traits<R>::template write_element<"<<i<<">(r)=vec_traits<A>::template read_element<"<<i<<">(a);" NL;
+            g<<TAB1 "write_vec_element<"<<i<<">(r,vec_traits<A>::template read_element<"<<i<<">(a));" NL;
         g<<
             TAB1 "return r;" NL
             TAB1 "}" NL
@@ -1371,7 +1368,7 @@ namespace
         std::ostream & g=out.stream();
         g<<
             "template <class A,class B>" NL
-            "BOOST_QVM_INLINE_OPERATIONS" NL
+            "BOOST_QVM_CONSTEXPR BOOST_QVM_INLINE_OPERATIONS" NL
             "typename lazy_enable_if_c<" NL
             TAB1 "mat_traits<A>::rows=="<<d<<" && mat_traits<A>::cols=="<<d<<" && is_scalar<B>::value," NL
             TAB1 "deduce_mat2<A,B,mat_traits<A>::rows,mat_traits<A>::cols> >::type" NL
@@ -1391,16 +1388,16 @@ namespace
         for( int i=0; i!=d; ++i )
             for( int j=0; j!=d; ++j )
                 {
-                g<<TAB1 "mat_traits<R>::template write_element<"<<i<<','<<j<<">(r)="<<(((i+j)&1)?'-':' ')<<"f*";
+                g<<TAB1 "write_mat_element<"<<i<<','<<j<<">(r,"<<(((i+j)&1)?'-':' ')<<"f*";
                 determinant_impl(g,d-1,del_row_col(del_row_col('a'),j,i));
-                g<<";" NL;
+                g<<");" NL;
                 }
         g<<
             TAB1 "return r;" NL
             TAB1 "}" NL
             NL
             "template <class A>" NL
-            "BOOST_QVM_INLINE_OPERATIONS" NL
+            "BOOST_QVM_CONSTEXPR BOOST_QVM_INLINE_OPERATIONS" NL
             "typename lazy_enable_if_c<" NL
             TAB1 "mat_traits<A>::rows=="<<d<<" && mat_traits<A>::cols=="<<d<<"," NL
             TAB1 "deduce_mat<A> >::type" NL
@@ -1502,7 +1499,7 @@ namespace
             TAB1 "R r;" NL
             ;
         for( int i=0; i!=d; ++i )
-            g<<TAB1 "vec_traits<R>::template write_element<"<<i<<">(r)=a"<<i<<"*rm;" NL;
+            g<<TAB1 "write_vec_element<"<<i<<">(r,a"<<i<<"*rm);" NL;
         g<<
             TAB1 "return r;" NL
             TAB1 "}" NL
@@ -1514,7 +1511,7 @@ namespace
             TAB1 "}" NL
             NL
             "template <class A>" NL
-            "BOOST_QVM_INLINE_OPERATIONS" NL
+            "BOOST_QVM_CONSTEXPR BOOST_QVM_INLINE_OPERATIONS" NL
             "typename enable_if_c<" NL
             TAB1 "vec_traits<A>::dim=="<<d<<"," NL
             TAB1 "void>::type" NL
@@ -1538,7 +1535,7 @@ namespace
             TAB1 "T const rm=scalar_traits<T>::value(1)/sqrt(m2);" NL
             ;
         for( int i=0; i!=d; ++i )
-            g<<TAB1 "vec_traits<A>::template write_element<"<<i<<">(a)*=rm;" NL;
+            g<<TAB1 "write_vec_element<"<<i<<">(a,vec_traits<A>::template read_element<"<<i<<">(a)*rm);" NL;
         g<<TAB1 "}" NL;
         defined(g,d,"normalize",suffix);
         }
@@ -1690,7 +1687,7 @@ namespace
     swizzle( output_file & out, int d )
         {
         assert(d>1);
-        out.require_include(INCLUDE_INLINE);
+        out.require_include(INCLUDE_CONFIG);
         out.require_include(INCLUDE_SWIZZLE_TRAITS);
         out.require_include(INCLUDE_ENABLE_IF);
         std::ostream & g=out.stream();
@@ -1763,29 +1760,29 @@ namespace
             mr_op_ma_mb_same_size(f,d,d,"operator-","-","mm");
             mr_op_ma_mb_same_size(f,d,1,"operator-","-","mm");
             mr_op_ma_mb_same_size(f,1,d,"operator-","-","mm");
-            ma_op_ma_mb_same_size(f,d,d,"operator+=","+=","mm");
-            ma_op_ma_mb_same_size(f,d,1,"operator+=","+=","mm");
-            ma_op_ma_mb_same_size(f,1,d,"operator+=","+=","mm");
-            ma_op_ma_mb_same_size(f,d,d,"operator-=","-=","mm");
-            ma_op_ma_mb_same_size(f,d,1,"operator-=","-=","mm");
-            ma_op_ma_mb_same_size(f,1,d,"operator-=","-=","mm");
+            ma_op_ma_mb_same_size(f,d,d,"operator+=","+","mm");
+            ma_op_ma_mb_same_size(f,d,1,"operator+=","+","mm");
+            ma_op_ma_mb_same_size(f,1,d,"operator+=","+","mm");
+            ma_op_ma_mb_same_size(f,d,d,"operator-=","-","mm");
+            ma_op_ma_mb_same_size(f,d,1,"operator-=","-","mm");
+            ma_op_ma_mb_same_size(f,1,d,"operator-=","-","mm");
             mr_op_ma_sb(f,d,d,"operator*","*","ms");
             mr_op_sa_mb(f,d,d,"operator*","*","sm");
             mr_op_ma_sb(f,d,1,"operator*","*","ms");
             mr_op_sa_mb(f,d,1,"operator*","*","sm");
             mr_op_ma_sb(f,1,d,"operator*","*","ms");
             mr_op_sa_mb(f,1,d,"operator*","*","sm");
-            ma_op_ma_sb(f,d,d,"operator*=","*=","ms");
-            ma_op_ma_sb(f,d,1,"operator*=","*=","ms");
-            ma_op_ma_sb(f,1,d,"operator*=","*=","ms");
+            ma_op_ma_sb(f,d,d,"operator*=","*","ms");
+            ma_op_ma_sb(f,d,1,"operator*=","*","ms");
+            ma_op_ma_sb(f,1,d,"operator*=","*","ms");
             mr_op_ma_sb(f,d,d,"operator/","/","ms");
             mr_op_sa_mb(f,d,d,"operator/","/","sm");
             mr_op_ma_sb(f,d,1,"operator/","/","ms");
             mr_op_sa_mb(f,d,1,"operator/","/","sm");
             mr_op_ma_sb(f,1,d,"operator/","/","ms");
-            ma_op_ma_sb(f,d,d,"operator/=","/=","ms");
-            ma_op_ma_sb(f,d,1,"operator/=","/=","ms");
-            ma_op_ma_sb(f,1,d,"operator/=","/=","ms");
+            ma_op_ma_sb(f,d,d,"operator/=","/","ms");
+            ma_op_ma_sb(f,d,1,"operator/=","/","ms");
+            ma_op_ma_sb(f,1,d,"operator/=","/","ms");
             mr_convert_to_ma(f,d,d,"m");
             mr_convert_to_ma(f,d,1,"m");
             mr_convert_to_ma(f,1,d,"m");
@@ -1826,13 +1823,13 @@ namespace
                 }
             vr_op_va_vb_same_size(f,d,"operator+","+","vv");
             vr_op_va_vb_same_size(f,d,"operator-","-","vv");
-            va_op_va_vb_same_size(f,d,"operator+=","+=","vv");
-            va_op_va_vb_same_size(f,d,"operator-=","-=","vv");
+            va_op_va_vb_same_size(f,d,"operator+=","+","vv");
+            va_op_va_vb_same_size(f,d,"operator-=","-","vv");
             vr_op_va_sb(f,d,"operator*","*","vs");
             vr_op_sa_vb(f,d,"operator*","*","sv");
-            va_op_va_sb(f,d,"operator*=","*=","vs");
+            va_op_va_sb(f,d,"operator*=","*","vs");
             vr_op_va_sb(f,d,"operator/","/","vs");
-            va_op_va_sb(f,d,"operator/=","/=","vs");
+            va_op_va_sb(f,d,"operator/=","/","vs");
             vr_convert_to_va(f,d,"v");
             bool_eq_va_vb(f,d,"vv");
             bool_neq_va_vb(f,d,"vv");
