@@ -13,6 +13,7 @@
 #include <fstream>
 
 #include <boost/iterator/iterator_traits.hpp>
+#include <boost/format.hpp>
 
 #include <boost/wave/wave_config.hpp>
 #include <boost/wave/language_support.hpp>
@@ -420,9 +421,9 @@ lexertl<Iterator, Position>::init_data_pp_number[INIT_DATA_PP_NUMBER_SIZE] =
 
 // C++11 specific token definitions
 
-#define T_EXTCHARLIT      token_id(T_CHARLIT|AltTokenType)
-#define T_EXTSTRINGLIT    token_id(T_STRINGLIT|AltTokenType)
-#define T_EXTRAWSTRINGLIT token_id(T_RAWSTRINGLIT|AltTokenType)
+constexpr token_id T_EXTCHARLIT = T_CHARLIT | AltTokenType;
+constexpr token_id T_EXTSTRINGLIT = T_STRINGLIT | AltTokenType;
+constexpr token_id T_EXTRAWSTRINGLIT = T_RAWSTRINGLIT | AltTokenType;
 
 template <typename Iterator, typename Position>
 typename lexertl<Iterator, Position>::lexer_data const
@@ -773,15 +774,11 @@ public:
                     case T_ANYCTRL:
                         // matched some unexpected character
                         {
-                            // 21 is the max required size for a 64 bit integer
-                            // represented as a string
-                            char buffer[22];
                             string_type msg("invalid character in input stream: '0x");
 
-                            // for some systems sprintf is in namespace std
-                            using namespace std;
-                            sprintf(buffer, "%02x'", token_val[0]);
-                            msg += buffer;
+                            std::string buffer = (boost::format("%02x'") % token_val[0]).str();
+
+                            msg += buffer.c_str();
                             BOOST_WAVE_LEXER_THROW(
                                 wave::cpplexer::lexing_exception,
                                 generic_lexing_error,
@@ -839,10 +836,6 @@ lexer::lexertl<
 #undef INIT_MACRO_DATA_SIZE
 #undef T_ANYCTRL
 
-#undef T_EXTCHARLIT
-#undef T_EXTSTRINGLIT
-#undef T_EXTRAWSTRINGLIT
-
 ///////////////////////////////////////////////////////////////////////////////
 //
 //  The new_lexer_gen<>::new_lexer function (declared in lexertl_interface.hpp)
